@@ -12,6 +12,7 @@ from __future__ import absolute_import
 import time
 import logging
 
+from django import VERSION as DJANGO_VERSION
 from django.conf import settings
 from django.core.exceptions import SuspiciousOperation
 from django.http import HttpRequest
@@ -30,13 +31,21 @@ from raven.contrib.django.utils import get_data_from_template, get_host
 from raven.contrib.django.middleware import SentryMiddleware
 from raven.utils.compat import string_types, binary_type, iterlists
 from raven.contrib.django.resolver import RouteResolver
-from raven.utils.wsgi import get_headers, get_environ
+from raven.utils.wsgi import get_headers, get_environ, get_client_ip
 from raven.utils import once
 from raven import breadcrumbs
 
 __all__ = ('DjangoClient',)
 
 
+if DJANGO_VERSION < (1, 10):
+    def is_authenticated(request_user):
+        return request_user.is_authenticated()
+else:
+    def is_authenticated(request_user):
+        return request_user.is_authenticated
+
+
 class _FormatConverter(object):
 
     def __init__(self, param_mapping):
@@ -74,6 +83,23 @@ def format_sql(sql, params):
     return sql, rv
 
 
+def record_sql(vendor, alias, start, duration, sql, params):
+    def processor(data):
+        real_sql, real_params = format_sql(sql, params)
+        if real_params:
+            try:
+                real_sql = real_sql % tuple(real_params)
+            except TypeError:
+                pass
+        # maybe category to 'django.%s.%s' % (vendor, alias or
+        #   'default') ?
+        data.update({
+            'message': real_sql,
+            'category': 'query',
+        })
+    breadcrumbs.record(processor=processor)
+
+
 @once
 def install_sql_hook():
     """If installed this causes Django's queries to be captured."""
@@ -90,19 +116,6 @@ def install_sql_hook():
         # trickery would have to look different but I can't be bothered.
         return
 
-    def record_sql(vendor, alias, start, duration, sql, params):
-        def processor(data):
-            real_sql, real_params = format_sql(sql, params)
-            if real_params:
-                real_sql = real_sql % tuple(real_params)
-            # maybe category to 'django.%s.%s' % (vendor, alias or
-            #   'default') ?
-            data.update({
-                'message': real_sql,
-                'category': 'query',
-            })
-        breadcrumbs.record(processor=processor)
-
     def record_many_sql(vendor, alias, start, sql, param_list):
         duration = time.time() - start
         for params in param_list:
@@ -142,19 +155,19 @@ class DjangoClient(Client):
     def install_sql_hook(self):
         install_sql_hook()
 
-    def get_user_info(self, user):
+    def get_user_info(self, request):
+
+        user_info = {
+            'ip_address': get_client_ip(request.META),
+        }
+        user = getattr(request, 'user', None)
+        if user is None:
+            return user_info
+
         try:
-            if hasattr(user, 'is_authenticated'):
-                # is_authenticated was a method in Django < 1.10
-                if callable(user.is_authenticated):
-                    authenticated = user.is_authenticated()
-                else:
-                    authenticated = user.is_authenticated
-                if not authenticated:
-                    return None
-
-            user_info = {}
-
+            authenticated = is_authenticated(user)
+            if not authenticated:
+                return user_info
             user_info['id'] = user.pk
 
             if hasattr(user, 'email'):
@@ -164,22 +177,22 @@ class DjangoClient(Client):
                 user_info['username'] = user.get_username()
             elif hasattr(user, 'username'):
                 user_info['username'] = user.username
-
-            return user_info
         except Exception:
             # We expect that user objects can be somewhat broken at times
             # and try to just handle as much as possible and ignore errors
             # as good as possible here.
-            return None
+            pass
+
+        return user_info
 
     def get_data_from_request(self, request):
-        result = {}
+        rv = {}
+        self.update_data_from_request(request, rv)
+        return rv
 
-        user = getattr(request, 'user', None)
-        if user is not None:
-            user_info = self.get_user_info(user)
-            if user_info:
-                result['user'] = user_info
+    def update_data_from_request(self, request, result):
+        if result.get('user') is None:
+            result['user'] = self.get_user_info(request)
 
         try:
             uri = request.build_absolute_uri()
@@ -227,8 +240,6 @@ class DjangoClient(Client):
             }
         })
 
-        return result
-
     def build_msg(self, *args, **kwargs):
         data = super(DjangoClient, self).build_msg(*args, **kwargs)
 
@@ -257,7 +268,7 @@ class DjangoClient(Client):
         return data
 
     def capture(self, event_type, request=None, **kwargs):
-        if 'data' not in kwargs:
+        if kwargs.get('data') is None:
             kwargs['data'] = data = {}
         else:
             data = kwargs['data']
@@ -267,7 +278,7 @@ class DjangoClient(Client):
 
         is_http_request = isinstance(request, HttpRequest)
         if is_http_request:
-            data.update(self.get_data_from_request(request))
+            self.update_data_from_request(request, data)
 
         if kwargs.get('exc_info'):
             exc_value = kwargs['exc_info'][1]
